require "path_rules/version"
require 'active_support/core_ext/date'

module PathRules
  # == パス名のルールを表現するクラス。
  class Template
    # @param tmplate_string [String] パス名のテンプレート%sなどのformatを指定できる。
    # @example Template.new('/tmp/log/%s/access.log')
    def initialize(tmplate_string)
      @tmplate_string = tmplate_string
    end

    # テンプレートを使って具体的なパス名を生成する。
    # @param *infix [String] パス名に埋め込む文字列
    # @example 例
    #   tmplate = Template.new('/tmp/log/%s/access.log')
    #   tmplate.instance('2015-01-01') #=> '/tmp/log/2015-01-01/access.log'
    def instance(*infix)
      format(@tmplate_string, *infix)
    end
  end

  # == 日付によるprefix, suffix, infixなどを表現するためのクラス
  class DateInfix
    #
    # 各種メソッドで日付を表現する時のフォーマット
    #
    OUTPUT_FORMAT = '%Y-%m-%d'

    # @param date [String] infixとして使いたい日付。'YYYY-MM-DD'形式。
    def initialize(date)
      check_argument_format!(date)

      @date = Date.strptime(date, '%Y-%m-%d')
    end

    def to_s
      @date.strftime(OUTPUT_FORMAT)
    end

    # @return [String] initializeに使った日付の月初日。
    def beginning_of_month
      @date.beginning_of_month.strftime(OUTPUT_FORMAT)
    end

    private

    # @param date [String] initializeの引数となった文字列。
    def check_argument_format!(date)
      unless date =~ /\d{4}-\d{2}-\d{2}/
        raise ArgumentError.new('引数はYYYY-MM-DD形式の文字列で与えてください。')
      end
    end
  end
end
